/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




/* GPL_HEADER */
package org.greenstone.gatherer.cdm;
/**************************************************************************************
 * Title:        Gatherer
 * Description:  The Gatherer: a tool for gathering and enriching a digital collection.
 * Copyright:    Copyright (c) 2001
 * Company:      The University of Waikato
 * Written:        /05/02
 * Revised:      22/08/02 Revamped, Optimized and Commented.
 **************************************************************************************/
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CollectionDesignManager;
import org.greenstone.gatherer.cdm.CollectionMeta;
import org.greenstone.gatherer.cdm.Index;
import org.greenstone.gatherer.cdm.Language;
import org.greenstone.gatherer.gui.EditorDialog;
import org.greenstone.gatherer.util.Utility;
/** This class is responsible for maintaining a list of assigned collection level metadata, and for allows manipulations on the aforementioned data.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
// ####################################################################################
// Optimization                               Saving
// ####################################################################################
// Removed Vector and Hashtable               + Memory, + Processor
// ####################################################################################
public class CollectionMetaManager
    extends DefaultListModel {
    /** A reference to the cdm manager so we can access indexes and languages. */
    private CollectionDesignManager manager = null;
    /** A reference to ourself so that inner classes can use us as a model. */
    private DefaultListModel model = null;
    /** A reference to the Gatherer. */
    private Gatherer gatherer = null;
    /** The language the most recent metadata returned was in. */
    private Language current_language = null;
    /** We can't safely parse metadata commands until after all the other commands have been parsed, so we store commands here for now. */
    private ArrayList unresolved_commands = null;
    /** Constructor.
     * @param gatherer A reference to the <strong>Gatherer</strong>.
     * @param manager A reference to the <strong>CollectionDesignManager</strong> for access to other configuration managers.
     */
    public CollectionMetaManager(Gatherer gatherer, CollectionDesignManager manager) {
	super();
	this.gatherer = gatherer;
	this.manager = manager;
	this.model = this;
	this.unresolved_commands = new ArrayList();
    }
    /** Method to add a new piece of metadata.
	  * @param metadata The new <strong>CollectionMeta</strong>.
	  */
    public void addMetadata(CollectionMeta metadata) {
	CollectionMeta existing = getMetadata(metadata.getName().toString(), metadata.getLanguage(), false);
	if(existing != null) {
	    removeElement(existing);
	}
	addElement(metadata);
	gatherer.c_man.configurationChanged();
    }
    /** Retrieve the collectionextra metadata in the default language, if present.
	  * @return A <strong>CollectionMeta</strong> containing the collectionextra in the default language if present, or else the first collectionextra of any language, otherwise <i>null</i>.
	  * @see org.greenstone.gatherer.cdm.Language
	  */
    public CollectionMeta getCollectionExtra() {
	CollectionMeta result = getMetadata("collectionextra", manager.languages.getDefaultLanguage(), true);
	if(result == null) {
	    result = new CollectionMeta(manager, "collectionextra", manager.languages.getDefaultLanguage(), "");
	    addMetadata(result);
	}
	return result;
    }
    /** Retrieve the collectionname metadata in the default language, if present.
	  * @return A <strong>CollectionMeta</strong> containing the collectionname in the default language if present, or else the first collectionname of any language, otherwise <i>null</i>.
	  * @see org.greenstone.gatherer.cdm.Language
	  */
    public CollectionMeta getCollectionName() {
	return getMetadata("collectionname", manager.languages.getDefaultLanguage(), true);
    }
    /** Retrieve the iconcollection metadata in the default language, if present.
	  * @return A <strong>CollectionMeta</strong> containing the iconcollection in the default language if present, or else the first iconcollection of any language, otherwise <i>null</i>.
	  * @see org.greenstone.gatherer.cdm.Language
	  */
    public CollectionMeta getIconCollection() {
	CollectionMeta result = getMetadata("iconcollection", manager.languages.getDefaultLanguage(), true);
	if(result == null) {
	    result = new CollectionMeta(manager, "iconcollection", manager.languages.getDefaultLanguage(), "");
	    addMetadata(result);
	}
	return result;
    }
    /** Retrieve the iconcollection metadata in the default language, if present.
	  * @return A <strong>CollectionMeta</strong> containing the iconcollectionsmall in the default language if present, or else the first iconcollectionsmall of any language, otherwise <i>null</i>.
	  * @see org.greenstone.gatherer.cdm.Language
	  */
    public CollectionMeta getIconCollectionSmall() {
	CollectionMeta result = getMetadata("iconcollectionsmall", manager.languages.getDefaultLanguage(), true);
	if(result == null) {
	    result = new CollectionMeta(manager, "iconcollectionsmall", manager.languages.getDefaultLanguage(), "");
	    addMetadata(result);
	}
	return result;
    }
    /** Method to retrieve the list of metadata.
	  * @return An <strong>ArrayList</strong> containing the metadata.
	  */
    public ArrayList getMetadata() {
	ArrayList metadata = new ArrayList();
	for(int i = 0; i < size(); i++) {
	    metadata.add(get(i));
	}
	Collections.sort(metadata);
	return metadata;
    }
	 
    /** Retrieve all of the metadata for the given feature, regardless of language. */
    public ArrayList getMetadata(Object name) {
	ArrayList result = new ArrayList();
	for(int i = 0; i < size(); i++) {
	    CollectionMeta metadata = (CollectionMeta) get(i);
	    if(metadata.getName().equals(name)) {
		result.add(metadata);
	    }
	}
	return result;
    }
    /** Method to retrieve a certain piece of metadata based on its name and language.
	  * @param name the name of the metadata as an Object (as it may actually be a refernce to an Index or SubIndex)
	  * @param language The <strong>Language</strong> of the metadata.
	  * @param partial <i>true</i> to return the first partial match (ie matches name but not language).
	  * @return The <strong>CollectionMeta</strong> requested, or <i>null</i> if no such metadata.
	  */
    public CollectionMeta getMetadata(Object name, Language language, boolean partial) { 
	CollectionMeta partial_match = null;
	for(int i = 0; i < size(); i++) {
	    CollectionMeta metadata = (CollectionMeta) get(i);
	    Object metadata_name = metadata.getName();
	    // We test the case of an object match (ie Index to Index)...
	    if(metadata_name.equals(name)) {
		if (metadata.getLanguage().equals(language)) {
		    return metadata;
		}
		partial_match = metadata;
	    }
	    // But if that fails we also try a string match such that "section:dls.Title" == Index
	    if(metadata_name.toString().equals(name.toString())) {
		if (metadata.getLanguage().equals(language)) {
		    return metadata;
		}
		partial_match = metadata;
	    }
	}
	if(partial) {
	    return partial_match;
	}
	return null;
    }
    /** Method that attempts to parse a collection metadata command from the given text. If a command is parsed successfully it is immediately added to the the collections metadata.
	 * @param command The command text we wish to parse, as a <strong>String</strong>.
	 * @return A <i>boolean</i> which is <i>true</i> if a collection metadata command was successfully parsed, <i>false</i> otherwise.
	 * @see org.greenstone.gatherer.cdm.CollectionDesignManager
	 * @see org.greenstone.gatherer.cdm.CommandTokenizer
	 * @see org.greenstone.gatherer.cdm.Language
	 * @see org.greenstone.gatherer.cdm.LanguageManager
	 * @see org.greenstone.gatherer.util.Utility
	 */
    public boolean parse(String command, boolean finished) {
	String temp = command.toLowerCase();
	if(temp.startsWith("collectionmeta")) {
	    if(finished) {
		CommandTokenizer ct = new CommandTokenizer(command);
		if(ct.countTokens() >= 3) {
		    ct.nextToken(); // Throw away collectionmeta
		    Object key = ct.nextToken();
		    Language language = null;
		    String language_code = null;
		    String value = ct.nextToken();
		    // Arg. Remember a language token will be '[l=<code>]'
		    if(value.startsWith("[") && value.endsWith("]")) {
			language_code = value.substring(3, value.length() - 1);
			///ystem.err.println("Language code = " + language_code);
			value = ct.nextToken();
		    }
		    // Check if the key is an index, and if so retrieve it.
		    if(((String)key).startsWith(".")) {
			///ystem.err.println("Detected '.' prefix.");
			String key_str = (String)key;
			key = null;
			key_str = key_str.substring(1);
			Index ind = Index.parseIndexConfig(key_str, manager);
			if (ind != null) {
			    
			    key = manager.indexes.getIndex(ind.toString(false));
			}
			
			// If that didn't work, then it might be a subindex so have a bash at retrieving that instead.
			if(key == null) {
			    key = manager.subcollections.getSubIndex(key_str);
			    //if(key != null) {
				///ystem.err.println("Its a subindex.");
			    //}
			    //else {
				///ystem.err.println("Error Will Robinson.");
			    //}
			}
			//else {
			    ///ystem.err.println("Its an index.");
			//}
		    }
		    // An if we have a language code, retrieve its object too.
		    if(language_code != null) {
			language = manager.languages.getLanguage(language_code, false);
		    }
		    // Otherwise set language to the default language.
		    else {
			language = manager.languages.getDefaultLanguage();
		    }
		    if(key != null) {
			// Trim any "
			if(value.equals("\"\"")) {
			    value = "";
			}
			else {
			    if(value.startsWith("\"")) {
				value = value.substring(1);
			    }
			    if(value.endsWith("\"")) {
				value = value.substring(0, value.length() - 1);
			    }
			}
			CollectionMeta meta = new CollectionMeta(manager, key, language, Utility.decodeGreenstone(value));
			addMetadata(meta);
		    }
		    return true;
		}
	    }
	    else {
		unresolved_commands.add(command);
		return true;
	    }
	}
	return false;
    }
    /** Ensure that the values being showed are the most up to date. */
    public void refresh() {
	fireContentsChanged(this, 0, size());
    }

    /** Method to remove a piece of metadata.
	  * @param metadata The <strong>CollectionMeta</strong> you wish to remove.
	  */
    public void removeMetadata(CollectionMeta metadata) {
	removeElement(metadata);
	gatherer.c_man.configurationChanged();
    }
    /** Method which attempts to reparse obvious metadata commands which used unresovable references.
	  */
    public void reparseUnresolved() {
	for(int i = 0; i < unresolved_commands.size(); i++) {
	    parse((String)unresolved_commands.get(i), true);
	}
	// Regardless of if they work, clear the commands.
	unresolved_commands.clear();
    }
    /** Sets the value of a certain metadata.
	  * @param name The name of the metadata as a <strong>String</strong>.
	  * @param language The <strong>Language</strong> to use.
	  * @param value The value of the metadata also as a <strong>String</strong>.
	  */
    public void setMetadata(String name, Language language, String value) {
	addMetadata(new CollectionMeta(manager, name, language, value));
    }
    /** Method to produce the list of metadata in a string such as you would find in the collection configuration file.
	  * @return A <strong>String</strong> containing the list of collection metadata.
	  */
    public String toString() {
	// We sort the metadata first.
	ArrayList metadatum = new ArrayList();
	for(int i = 0; i < size(); i++) {
	    metadatum.add(get(i));
	}
	Collections.sort(metadatum, new CollectionMetaComparator());
	// Now we print them
	StringBuffer text = new StringBuffer("");
	for(int i = 0; i < metadatum.size(); i++) {
	    CollectionMeta data = (CollectionMeta) metadatum.get(i);
	    if(data.valid()) {
		text.append(data.toString());
	    }
	}
	metadatum = null;
	text.append("\n");
	return text.toString();
    }
    /** Overloaded to call get with both a key and an empty argument array.
	  * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
	  * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
	  */
    private String get(String key) {
	return get(key, null);
    }
    /** Used to retrieve a property value from the Locale specific ResourceBundle, based upon the key and arguments supplied. If the key cannot be found or if some other part of the call fails a default (English) error message is returned. <BR>
	  * Here the get recieves a second argument which is an array of Strings used to populate argument fields, denoted {<I>n</I>}, within the value String returned. Note that argument numbers greater than or equal to 32 are automatically mapped to the formatting String named Farg<I>n</I>.
	  * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
	  * @param args A <strong>String[]</strong> used to populate argument fields within the complete String.
	  * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.Dictionary
	  */
    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "CDM.CollectionMetaManager." + key;
	}
	return gatherer.dictionary.get(key, args);
    }

    private class CollectionMetaComparator
	implements Comparator {
	public int compare(Object o1, Object o2) {
	    String s1 = o1.toString();
	    String s2 = o2.toString();
	    if(s1.startsWith(".")) {
		return 1;
	    }
	    else if(s2.startsWith(".")) {
		return -1;
	    }
	    return o1.toString().compareTo(o2.toString());
	}
	public boolean equals(Object o2) {
	    return (compare(this, o2) == 0);
	}
    }
}
